"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Family(_ParentRestClient):
    """
    This class contains methods for the 'Families' webservices
    PATH: /{apiVersion}/families
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Family, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def update_acl(self, members, action, data=None, **options):
        """
        Update the set of permissions granted for the member.
        PATH: /{apiVersion}/families/acl/{members}/update

        :param dict data: JSON containing the parameters to add ACLs.
            (REQUIRED)
        :param str action: Action to be performed [ADD, SET, REMOVE or RESET].
            Allowed values: ['SET ADD REMOVE RESET'] (REQUIRED)
        :param str members: Comma separated list of user or group ids.
            (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str propagate: Propagate family permissions to related
            individuals and samples. Allowed values: ['NO YES
            YES_AND_VARIANT_VIEW']
        """

        options['action'] = action
        return self._post(category='families', resource='update', subcategory='acl', second_query_id=members, data=data, **options)

    def aggregation_stats(self, **options):
        """
        Fetch catalog family stats.
        PATH: /{apiVersion}/families/aggregationStats

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str id: Comma separated list family IDs up to a maximum of 100.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str name: Comma separated list family names up to a maximum of
            100. Also admits basic regular expressions using the operator '~',
            i.e. '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i'
            for case insensitive search.
        :param str uuid: Comma separated list family UUIDs up to a maximum of
            100.
        :param str members: Comma separated list of family members.
        :param int expected_size: Expected size of the family (number of
            members).
        :param str samples: Comma separated list of member's samples.
        :param str phenotypes: Comma separated list of phenotype ids or names.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str disorders: Comma separated list of disorder ids or names.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param bool deleted: Boolean to retrieve deleted entries.
        :param str internal_status: Filter by internal status.
        :param str status: Filter by status.
        :param str annotation: Annotation filters. Example:
            age>30;gender=FEMALE. For more information, please visit
            http://docs.opencb.org/display/opencga/AnnotationSets+1.4.0.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param str release: Release when it was created.
        :param int snapshot: Snapshot value (Latest version of the entry in
            the specified release).
        :param str field: Field to apply aggregation statistics to (or a list
            of fields separated by semicolons), e.g.:
            studies;type;numSamples[0..10]:1;format:sum(size).
        """

        return self._get(category='families', resource='aggregationStats', **options)

    def load_annotation_sets(self, variable_set_id, path, data=None, **options):
        """
        Load annotation sets from a TSV file.
        PATH: /{apiVersion}/families/annotationSets/load

        :param str path: Path where the TSV file is located in OpenCGA or
            where it should be located. (REQUIRED)
        :param str variable_set_id: Variable set ID or name. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool parents: Flag indicating whether to create parent
            directories if they don't exist (only when TSV file was not
            previously associated).
        :param str annotation_set_id: Annotation set id. If not provided,
            variableSetId will be used.
        :param dict data: JSON containing the 'content' of the TSV file if
            this has not yet been registered into OpenCGA.
        """

        options['variableSetId'] = variable_set_id
        options['path'] = path
        return self._post(category='families', resource='load', subcategory='annotationSets', data=data, **options)

    def create(self, data=None, **options):
        """
        Create family and the individual objects if they do not exist.
        PATH: /{apiVersion}/families/create

        :param dict data: JSON containing family information. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str members: Comma separated list of member ids to be
            associated to the created family.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._post(category='families', resource='create', data=data, **options)

    def distinct(self, field, **options):
        """
        Family distinct method.
        PATH: /{apiVersion}/families/distinct

        :param str field: Comma separated list of fields for which to obtain
            the distinct values. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str id: Comma separated list family IDs up to a maximum of 100.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str name: Comma separated list family names up to a maximum of
            100. Also admits basic regular expressions using the operator '~',
            i.e. '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i'
            for case insensitive search.
        :param str uuid: Comma separated list family UUIDs up to a maximum of
            100.
        :param str members: Comma separated list of family members.
        :param int expected_size: Expected size of the family (number of
            members).
        :param str samples: Comma separated list of member's samples.
        :param str phenotypes: Comma separated list of phenotype ids or names.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str disorders: Comma separated list of disorder ids or names.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param bool deleted: Boolean to retrieve deleted entries.
        :param str internal_status: Filter by internal status.
        :param str status: Filter by status.
        :param str annotation: Annotation filters. Example:
            age>30;gender=FEMALE. For more information, please visit
            http://docs.opencb.org/display/opencga/AnnotationSets+1.4.0.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param str release: Release when it was created.
        :param int snapshot: Snapshot value (Latest version of the entry in
            the specified release).
        """

        options['field'] = field
        return self._get(category='families', resource='distinct', **options)

    def search(self, **options):
        """
        Search families.
        PATH: /{apiVersion}/families/search

        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param bool flatten_annotations: Flatten the annotations?.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str id: Comma separated list family IDs up to a maximum of 100.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str name: Comma separated list family names up to a maximum of
            100. Also admits basic regular expressions using the operator '~',
            i.e. '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i'
            for case insensitive search.
        :param str uuid: Comma separated list family UUIDs up to a maximum of
            100.
        :param str members: Comma separated list of family members.
        :param int expected_size: Expected size of the family (number of
            members).
        :param str samples: Comma separated list of member's samples.
        :param str phenotypes: Comma separated list of phenotype ids or names.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str disorders: Comma separated list of disorder ids or names.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param bool deleted: Boolean to retrieve deleted entries.
        :param str internal_status: Filter by internal status.
        :param str status: Filter by status.
        :param str annotation: Annotation filters. Example:
            age>30;gender=FEMALE. For more information, please visit
            http://docs.opencb.org/display/opencga/AnnotationSets+1.4.0.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param str release: Release when it was created.
        :param int snapshot: Snapshot value (Latest version of the entry in
            the specified release).
        """

        return self._get(category='families', resource='search', **options)

    def acl(self, families, **options):
        """
        Returns the acl of the families. If member is provided, it will only
            return the acl for the member.
        PATH: /{apiVersion}/families/{families}/acl

        :param str families: Comma separated list of family IDs or names up to
            a maximum of 100. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str member: User or group id.
        :param bool silent: Boolean to retrieve all possible entries that are
            queried for, false to raise an exception whenever one of the
            entries looked for cannot be shown for whichever reason.
        """

        return self._get(category='families', resource='acl', query_id=families, **options)

    def delete(self, families, **options):
        """
        Delete existing families.
        PATH: /{apiVersion}/families/{families}/delete

        :param str families: Comma separated list of family ids. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._delete(category='families', resource='delete', query_id=families, **options)

    def info(self, families, **options):
        """
        Get family information.
        PATH: /{apiVersion}/families/{families}/info

        :param str families: Comma separated list of family IDs or names up to
            a maximum of 100. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param bool flatten_annotations: Flatten the annotations?.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str version: Comma separated list of family versions. 'all' to
            get all the family versions. Not supported if multiple family ids
            are provided.
        :param bool deleted: Boolean to retrieve deleted families.
        """

        return self._get(category='families', resource='info', query_id=families, **options)

    def update(self, families, data=None, **options):
        """
        Update some family attributes.
        PATH: /{apiVersion}/families/{families}/update

        :param str families: Comma separated list of family ids. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool update_roles: Update the member roles within the family.
        :param bool update_pedigree_graph: Update the family pedigree graph.
        :param str annotation_sets_action: Action to be performed if the array
            of annotationSets is being updated. Allowed values: ['ADD SET
            REMOVE']
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        :param dict data: body.
        """

        return self._post(category='families', resource='update', query_id=families, data=data, **options)

    def update_annotation_sets_annotations(self, family, annotation_set, data=None, **options):
        """
        Update annotations from an annotationSet.
        PATH: /{apiVersion}/families/{family}/annotationSets/{annotationSet}/annotations/update

        :param str annotation_set: AnnotationSet ID to be updated. (REQUIRED)
        :param str family: Family id. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str action: Action to be performed: ADD to add new annotations;
            REPLACE to replace the value of an already existing annotation; SET
            to set the new list of annotations removing any possible old
            annotations; REMOVE to remove some annotations; RESET to set some
            annotations to the default value configured in the corresponding
            variables of the VariableSet if any. Allowed values: ['ADD SET
            REMOVE RESET REPLACE']
        :param dict data: Json containing the map of annotations when the
            action is ADD, SET or REPLACE, a json with only the key 'remove'
            containing the comma separated variables to be removed as a value
            when the action is REMOVE or a json with only the key 'reset'
            containing the comma separated variables that will be set to the
            default value when the action is RESET.
        """

        return self._post(category='families', resource='annotations/update', query_id=family, subcategory='annotationSets', second_query_id=annotation_set, data=data, **options)




"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Workflow(_ParentRestClient):
    """
    This class contains methods for the 'Workflows' webservices
    PATH: /{apiVersion}/workflows
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Workflow, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def update_acl(self, members, action, data=None, **options):
        """
        Update the set of workflow permissions granted for the member.
        PATH: /{apiVersion}/workflows/acl/{members}/update

        :param dict data: JSON containing the parameters to update the
            permissions. (REQUIRED)
        :param str action: Action to be performed [ADD, SET, REMOVE or RESET].
            Allowed values: ['SET ADD REMOVE RESET'] (REQUIRED)
        :param str members: Comma separated list of user or group ids.
            (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        options['action'] = action
        return self._post(category='workflows', resource='update', subcategory='acl', second_query_id=members, data=data, **options)

    def create(self, data=None, **options):
        """
        Create a workflow.
        PATH: /{apiVersion}/workflows/create

        :param dict data: JSON containing workflow information. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._post(category='workflows', resource='create', data=data, **options)

    def distinct(self, field, **options):
        """
        Workflow distinct method.
        PATH: /{apiVersion}/workflows/distinct

        :param str field: Comma separated list of fields for which to obtain
            the distinct values. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str id: Comma separated list of workflow IDs up to a maximum of
            100. Also admits basic regular expressions using the operator '~',
            i.e. '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i'
            for case insensitive search.
        :param str name: Comma separated list of workflow names up to a
            maximum of 100. Also admits basic regular expressions using the
            operator '~', i.e. '~{perl-regex}' e.g. '~value' for case
            sensitive, '~/value/i' for case insensitive search.
        :param str uuid: Comma separated list of workflow UUIDs up to a
            maximum of 100.
        :param str tags: Comma separated list of tags.
        :param bool draft: Boolean field indicating whether the workflow is a
            draft or not.
        :param str internal.registration_user_id: UserId that created the
            workflow.
        :param str manager.id: Id of the workflow system (Allowed values:
            NEXTFLOW).
        :param str type: Workflow type. Allowed types:
            [CLINICAL_INTERPRETATION, SECONDARY_ANALYSIS, RESEARCH or OTHER].
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param str release: Release when it was created.
        :param int snapshot: Snapshot value (Latest version of the entry in
            the specified release).
        :param bool deleted: Boolean to retrieve deleted entries.
        """

        options['field'] = field
        return self._get(category='workflows', resource='distinct', **options)

    def import_workflow(self, data=None, **options):
        """
        Import a workflow.
        PATH: /{apiVersion}/workflows/import

        :param dict data: Repository parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._post(category='workflows', resource='import', data=data, **options)

    def run(self, data=None, **options):
        """
        Execute a workflow analysis.
        PATH: /{apiVersion}/workflows/run

        :param dict data: NextFlow run parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='workflows', resource='run', data=data, **options)

    def search(self, **options):
        """
        Workflow search method.
        PATH: /{apiVersion}/workflows/search

        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str id: Comma separated list of workflow IDs up to a maximum of
            100. Also admits basic regular expressions using the operator '~',
            i.e. '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i'
            for case insensitive search.
        :param str name: Comma separated list of workflow names up to a
            maximum of 100. Also admits basic regular expressions using the
            operator '~', i.e. '~{perl-regex}' e.g. '~value' for case
            sensitive, '~/value/i' for case insensitive search.
        :param str uuid: Comma separated list of workflow UUIDs up to a
            maximum of 100.
        :param str tags: Comma separated list of tags.
        :param bool draft: Boolean field indicating whether the workflow is a
            draft or not.
        :param str internal.registration_user_id: UserId that created the
            workflow.
        :param str manager.id: Id of the workflow system (Allowed values:
            NEXTFLOW).
        :param str type: Workflow type. Allowed types:
            [CLINICAL_INTERPRETATION, SECONDARY_ANALYSIS, RESEARCH or OTHER].
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param str release: Release when it was created.
        :param int snapshot: Snapshot value (Latest version of the entry in
            the specified release).
        :param bool deleted: Boolean to retrieve deleted entries.
        """

        return self._get(category='workflows', resource='search', **options)

    def update(self, workflow_id, data=None, **options):
        """
        Update some workflow attributes.
        PATH: /{apiVersion}/workflows/{workflowId}/update

        :param str workflow_id: Comma separated list workflow IDs or UUIDs up
            to a maximum of 100. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        :param dict data: body.
        """

        return self._post(category='workflows', resource='update', query_id=workflow_id, data=data, **options)

    def acl(self, workflows, **options):
        """
        Returns the acl of the workflows. If member is provided, it will only
            return the acl for the member.
        PATH: /{apiVersion}/workflows/{workflows}/acl

        :param str workflows: Comma separated list workflow IDs or UUIDs up to
            a maximum of 100. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str member: User or group id.
        :param bool silent: Boolean to retrieve all possible entries that are
            queried for, false to raise an exception whenever one of the
            entries looked for cannot be shown for whichever reason.
        """

        return self._get(category='workflows', resource='acl', query_id=workflows, **options)

    def delete(self, workflows, **options):
        """
        Delete workflows.
        PATH: /{apiVersion}/workflows/{workflows}/delete

        :param str workflows: Comma separated list workflow IDs or UUIDs up to
            a maximum of 100. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._delete(category='workflows', resource='delete', query_id=workflows, **options)

    def info(self, workflows, **options):
        """
        Get workflow information.
        PATH: /{apiVersion}/workflows/{workflows}/info

        :param str workflows: Comma separated list sample IDs or UUIDs up to a
            maximum of 100. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str version: Comma separated list of workflow versions. 'all'
            to get all the workflow versions. Not supported if multiple
            workflow ids are provided.
        :param bool deleted: Boolean to retrieve deleted entries.
        """

        return self._get(category='workflows', resource='info', query_id=workflows, **options)




"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Meta(_ParentRestClient):
    """
    This class contains methods for the 'Meta' webservices
    PATH: /{apiVersion}/meta
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Meta, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def about(self, **options):
        """
        Returns info about current OpenCGA code.
        PATH: /{apiVersion}/meta/about
        """

        return self._get(category='meta', resource='about', **options)

    def api(self, **options):
        """
        API.
        PATH: /{apiVersion}/meta/api

        :param str category: List of categories to get API from.
        """

        return self._get(category='meta', resource='api', **options)

    def fail(self, **options):
        """
        Ping Opencga webservices.
        PATH: /{apiVersion}/meta/fail
        """

        return self._get(category='meta', resource='fail', **options)

    def model(self, **options):
        """
        Opencga model webservices.
        PATH: /{apiVersion}/meta/model

        :param str model: Model description.
        """

        return self._get(category='meta', resource='model', **options)

    def openapi(self, **options):
        """
        Opencga openapi json.
        PATH: /{apiVersion}/meta/openapi

        :param str token: List of categories to get API from.
        """

        return self._get(category='meta', resource='openapi', **options)

    def ping(self, **options):
        """
        Ping Opencga webservices.
        PATH: /{apiVersion}/meta/ping
        """

        return self._get(category='meta', resource='ping', **options)

    def status(self, **options):
        """
        Database status.
        PATH: /{apiVersion}/meta/status
        """

        return self._get(category='meta', resource='status', **options)




"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Alignment(_ParentRestClient):
    """
    This class contains methods for the 'Analysis - Alignment' webservices
    PATH: /{apiVersion}/analysis/alignment
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Alignment, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def run_bwa(self, data=None, **options):
        """
        BWA is a software package for mapping low-divergent sequences against
            a large reference genome.
        PATH: /{apiVersion}/analysis/alignment/bwa/run

        :param dict data: BWA parameters. (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/bwa', data=data, **options)

    def run_coverage_index(self, data=None, **options):
        """
        Compute the coverage from a given BAM alignment file, e.g., create a
            .bw file from a .bam file.
        PATH: /{apiVersion}/analysis/alignment/coverage/index/run

        :param dict data: Coverage computation parameters. (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/coverage/index', data=data, **options)

    def coverage_qc_gene_coverage_stats_run(self, data=None, **options):
        """
        Compute gene coverage stats for a given alignment file and a list of
            genes.
        PATH: /{apiVersion}/analysis/alignment/coverage/qc/geneCoverageStats/run

        :param dict data: Gene coverage stats parameters for a given BAM file
            and a list of genes. (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/coverage/qc/geneCoverageStats', data=data, **options)

    def query_coverage(self, file, **options):
        """
        Query the coverage of an alignment file for regions or genes.
        PATH: /{apiVersion}/analysis/alignment/coverage/query

        :param str file: File ID. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str region: Comma separated list of regions 'chr:start-end,
            e.g.: 2,3:63500-65000.
        :param str gene: Comma separated list of genes, e.g.: BCRA2,TP53.
        :param int offset: Offset to extend the region, gene or exon at up and
            downstream.
        :param bool only_exons: Only exons are taking into account when genes
            are specified.
        :param str range: Range of coverage values to be reported. Minimum and
            maximum values are separated by '-', e.g.: 20-40 (for coverage
            values greater or equal to 20 and less or equal to 40). A single
            value means to report coverage values less or equal to that value.
        :param int window_size: Window size for the region coverage (if a
            coverage range is provided, window size must be 1).
        :param bool split_results: Split results into regions (or gene/exon
            regions).
        """

        options['file'] = file
        return self._get(category='analysis', resource='query', subcategory='alignment/coverage', **options)

    def ratio_coverage(self, file1, file2, **options):
        """
        Compute coverage ratio from file #1 vs file #2, (e.g. somatic vs
            germline).
        PATH: /{apiVersion}/analysis/alignment/coverage/ratio

        :param str file2: Input file #2 (e.g. germline file). (REQUIRED)
        :param str file1: Input file #1 (e.g. somatic file). (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool skip_log2: Do not apply Log2 to normalise the coverage
            ratio.
        :param str region: Comma separated list of regions 'chr:start-end,
            e.g.: 2,3:63500-65000.
        :param str gene: Comma separated list of genes, e.g.: BCRA2,TP53.
        :param int offset: Offset to extend the region, gene or exon at up and
            downstream.
        :param bool only_exons: Only exons are taking into account when genes
            are specified.
        :param int window_size: Window size for the region coverage (if a
            coverage range is provided, window size must be 1).
        :param bool split_results: Split results into regions (or gene/exon
            regions).
        """

        options['file1'] = file1
        options['file2'] = file2
        return self._get(category='analysis', resource='ratio', subcategory='alignment/coverage', **options)

    def stats_coverage(self, file, gene, **options):
        """
        Compute coverage stats per transcript for a list of genes.
        PATH: /{apiVersion}/analysis/alignment/coverage/stats

        :param str gene: Comma separated list of genes, e.g.: BCRA2,TP53.
            (REQUIRED)
        :param str file: File ID. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param int threshold: Only regions whose coverage depth is under this
            threshold will be reported.
        """

        options['file'] = file
        options['gene'] = gene
        return self._get(category='analysis', resource='stats', subcategory='alignment/coverage', **options)

    def run_deeptools(self, data=None, **options):
        """
        Deeptools is a suite of python tools particularly developed for the
            efficient analysis of high-throughput sequencing data, such as
            ChIP-seq, RNA-seq or MNase-seq.
        PATH: /{apiVersion}/analysis/alignment/deeptools/run

        :param dict data: Deeptools parameters. Supported Deeptools commands:
            bamCoverage, bamCompare. (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/deeptools', data=data, **options)

    def run_fastqc(self, data=None, **options):
        """
        A high throughput sequence QC analysis tool.
        PATH: /{apiVersion}/analysis/alignment/fastqc/run

        :param dict data: FastQC parameters. (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/fastqc', data=data, **options)

    def run_index(self, data=None, **options):
        """
        Index a given alignment file BAM/CRAM, e.g., create a .bai file from a
            .bam file.
        PATH: /{apiVersion}/analysis/alignment/index/run

        :param dict data: Alignment index params. (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/index', data=data, **options)

    def run_picard(self, data=None, **options):
        """
        Picard is a set of command line tools (in Java) for manipulating
            high-throughput sequencing (HTS) data and formats such as
            SAM/BAM/CRAM and VCF. Supported Picard commands: CollectHsMetrics,
            CollectWgsMetrics, BedToIntervalList.
        PATH: /{apiVersion}/analysis/alignment/picard/run

        :param dict data: Picard parameters. Supported Picard commands:
            CollectHsMetrics, CollectWgsMetrics, BedToIntervalList. (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/picard', data=data, **options)

    def run_qc(self, data=None, **options):
        """
        Compute quality control (QC) metrics for a given alignment file:
            samtools stats, samtools flag stats and FastQC metrics.
        PATH: /{apiVersion}/analysis/alignment/qc/run

        :param dict data: Alignment quality control (QC) parameters. It
            computes: stats, flag stats and fastqc metrics. The BAM file ID is
            mandatory and in order to skip some metrics, use the following
            keywords (separated by commas): stats, flagstats, fastqc.
            (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/qc', data=data, **options)

    def query(self, file, **options):
        """
        Search over indexed alignments.
        PATH: /{apiVersion}/analysis/alignment/query

        :param str file: File ID. (REQUIRED)
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str region: Comma separated list of regions 'chr:start-end,
            e.g.: 2,3:63500-65000.
        :param str gene: Comma separated list of genes, e.g.: BCRA2,TP53.
        :param int offset: Offset to extend the region, gene or exon at up and
            downstream.
        :param bool only_exons: Only exons are taking into account when genes
            are specified.
        :param int min_mapping_quality: Minimum mapping quality.
        :param int max_num_mismatches: Maximum number of mismatches.
        :param int max_num_hits: Maximum number of hits.
        :param bool properly_paired: Return only properly paired alignments.
        :param int max_insert_size: Maximum insert size.
        :param bool skip_unmapped: Skip unmapped alignments.
        :param bool skip_duplicated: Skip duplicated alignments.
        :param bool region_contained: Return alignments contained within
            boundaries of region.
        :param bool force_m_d_field: Force SAM MD optional field to be set
            with the alignments.
        :param bool bin_qualities: Compress the nucleotide qualities by using
            8 quality levels.
        :param bool split_results: Split results into regions (or gene/exon
            regions).
        """

        options['file'] = file
        return self._get(category='analysis', resource='query', subcategory='alignment', **options)

    def run_samtools(self, data=None, **options):
        """
        Samtools is a program for interacting with high-throughput sequencing
            data in SAM, BAM and CRAM formats. Supported Samtools commands:
            sort, index, view, stats, flagstat, dict, faidx, depth,
            plot-bamstats.
        PATH: /{apiVersion}/analysis/alignment/samtools/run

        :param dict data: Samtools parameters. Supported Samtools commands:
            sort, index, view, stats, flagstat, dict, faidx, depth,
            plot-bamstats. (REQUIRED)
        :param str study: study.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='analysis', resource='run', subcategory='alignment/samtools', data=data, **options)




"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Job(_ParentRestClient):
    """
    This class contains methods for the 'Jobs' webservices
    PATH: /{apiVersion}/jobs
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Job, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def update_acl(self, members, action, data=None, **options):
        """
        Update the set of permissions granted for the member.
        PATH: /{apiVersion}/jobs/acl/{members}/update

        :param dict data: JSON containing the parameters to add ACLs.
            (REQUIRED)
        :param str action: Action to be performed [ADD, SET, REMOVE or RESET].
            Allowed values: ['SET ADD REMOVE RESET'] (REQUIRED)
        :param str members: Comma separated list of user or group ids.
            (REQUIRED)
        """

        options['action'] = action
        return self._post(category='jobs', resource='update', subcategory='acl', second_query_id=members, data=data, **options)

    def aggregation_stats(self, **options):
        """
        Fetch catalog job stats.
        PATH: /{apiVersion}/jobs/aggregationStats

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool other_studies: Flag indicating the entries being queried
            can belong to any related study, not just the primary one.
        :param str id: Comma separated list of job IDs up to a maximum of 100.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str uuid: Comma separated list of job UUIDs up to a maximum of
            100.
        :param str tool_id: Tool ID executed by the job. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str tool_type: Tool type executed by the job [OPERATION,
            ANALYSIS].
        :param str user_id: User that created the job.
        :param str priority: Priority of the job.
        :param str status: Filter by status.
        :param str internal_status: Filter by internal status.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param bool visited: Visited status of job.
        :param str tags: Job tags.
        :param str input: Comma separated list of file IDs used as input.
        :param str output: Comma separated list of file IDs used as output.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param str release: Release when it was created.
        :param bool deleted: Boolean to retrieve deleted entries.
        :param str field: Field to apply aggregation statistics to (or a list
            of fields separated by semicolons), e.g.:
            studies;type;numSamples[0..10]:1;format:sum(size).
        """

        return self._get(category='jobs', resource='aggregationStats', **options)

    def create(self, data=None, **options):
        """
        Register an executed job with POST method.
        PATH: /{apiVersion}/jobs/create

        :param dict data: job. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._post(category='jobs', resource='create', data=data, **options)

    def distinct(self, field, **options):
        """
        Job distinct method.
        PATH: /{apiVersion}/jobs/distinct

        :param str field: Comma separated list of fields for which to obtain
            the distinct values. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool other_studies: Flag indicating the entries being queried
            can belong to any related study, not just the primary one.
        :param str id: Comma separated list of job IDs up to a maximum of 100.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str uuid: Comma separated list of job UUIDs up to a maximum of
            100.
        :param str type: Job type (NATIVE, WORKFLOW, CUSTOM or WALKER).
        :param str tool_id: Tool ID executed by the job. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str tool_type: Tool type executed by the job [OPERATION,
            ANALYSIS].
        :param str tool.external_executor.id: Id of the external executor.
            This field is only applicable for jobs executed by an external
            executor.
        :param str parent_id: Job id that generated this job (if any).
        :param bool dry_run: Flag indicating that the job will be executed in
            dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param bool internal.kill_job_requested: Flag indicating that the user
            requested to kill the job.
        :param str user_id: User that created the job.
        :param str priority: Priority of the job.
        :param str status: Filter by status.
        :param str internal_status: Filter by internal status.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param bool visited: Visited status of job.
        :param str tags: Job tags.
        :param str input: Comma separated list of file IDs used as input.
        :param str output: Comma separated list of file IDs used as output.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param str release: Release when it was created.
        :param bool deleted: Boolean to retrieve deleted entries.
        """

        options['field'] = field
        return self._get(category='jobs', resource='distinct', **options)

    def retry(self, data=None, **options):
        """
        Relaunch a failed job.
        PATH: /{apiVersion}/jobs/retry

        :param dict data: job. (REQUIRED)
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._post(category='jobs', resource='retry', data=data, **options)

    def search(self, **options):
        """
        Job search method.
        PATH: /{apiVersion}/jobs/search

        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool other_studies: Flag indicating the entries being queried
            can belong to any related study, not just the primary one.
        :param str id: Comma separated list of job IDs up to a maximum of 100.
            Also admits basic regular expressions using the operator '~', i.e.
            '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i' for
            case insensitive search.
        :param str uuid: Comma separated list of job UUIDs up to a maximum of
            100.
        :param str type: Job type (NATIVE, WORKFLOW, CUSTOM or WALKER).
        :param str tool_id: Tool ID executed by the job. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str tool_type: Tool type executed by the job [OPERATION,
            ANALYSIS].
        :param str tool.external_executor.id: Id of the external executor.
            This field is only applicable for jobs executed by an external
            executor.
        :param str parent_id: Job id that generated this job (if any).
        :param bool dry_run: Flag indicating that the job will be executed in
            dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param bool internal.kill_job_requested: Flag indicating that the user
            requested to kill the job.
        :param str user_id: User that created the job.
        :param str priority: Priority of the job.
        :param str status: Filter by status.
        :param str internal_status: Filter by internal status.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param bool visited: Visited status of job.
        :param str tags: Job tags.
        :param str input: Comma separated list of file IDs used as input.
        :param str output: Comma separated list of file IDs used as output.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param str release: Release when it was created.
        :param bool deleted: Boolean to retrieve deleted entries.
        """

        return self._get(category='jobs', resource='search', **options)

    def build_tool(self, data=None, **options):
        """
        Execute an analysis from a custom binary.
        PATH: /{apiVersion}/jobs/tool/build

        :param dict data: body. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='jobs', resource='build', subcategory='tool', data=data, **options)

    def run_tool(self, data=None, **options):
        """
        Execute an analysis from a custom binary.
        PATH: /{apiVersion}/jobs/tool/run

        :param dict data: NextFlow run parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='jobs', resource='run', subcategory='tool', data=data, **options)

    def top(self, **options):
        """
        Provide a summary of the running jobs.
        PATH: /{apiVersion}/jobs/top

        :param int limit: Maximum number of jobs to be returned.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str internal_status: Filter by internal status.
        :param str priority: Priority of the job.
        :param str user_id: User that created the job.
        :param str tool_id: Tool ID executed by the job. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        """

        return self._get(category='jobs', resource='top', **options)

    def acl(self, jobs, **options):
        """
        Return the acl of the job. If member is provided, it will only return
            the acl for the member.
        PATH: /{apiVersion}/jobs/{jobs}/acl

        :param str jobs: Comma separated list of job IDs or UUIDs up to a
            maximum of 100. (REQUIRED)
        :param str member: User or group id.
        :param bool silent: Boolean to retrieve all possible entries that are
            queried for, false to raise an exception whenever one of the
            entries looked for cannot be shown for whichever reason.
        """

        return self._get(category='jobs', resource='acl', query_id=jobs, **options)

    def delete(self, jobs, **options):
        """
        Delete existing jobs.
        PATH: /{apiVersion}/jobs/{jobs}/delete

        :param str jobs: Comma separated list of job ids. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._delete(category='jobs', resource='delete', query_id=jobs, **options)

    def info(self, jobs, **options):
        """
        Get job information.
        PATH: /{apiVersion}/jobs/{jobs}/info

        :param str jobs: Comma separated list of job IDs or UUIDs up to a
            maximum of 100. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool deleted: Boolean to retrieve deleted jobs.
        """

        return self._get(category='jobs', resource='info', query_id=jobs, **options)

    def update(self, jobs, data=None, **options):
        """
        Update some job attributes.
        PATH: /{apiVersion}/jobs/{jobs}/update

        :param str jobs: Comma separated list of job IDs or UUIDs up to a
            maximum of 100. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        :param dict data: body.
        """

        return self._post(category='jobs', resource='update', query_id=jobs, data=data, **options)

    def kill(self, job, **options):
        """
        Send a signal to kill a pending or running job.
        PATH: /{apiVersion}/jobs/{job}/kill

        :param str job: Job ID or UUID. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._post(category='jobs', resource='kill', query_id=job, **options)

    def head_log(self, job, **options):
        """
        Show the first lines of a log file (up to a limit).
        PATH: /{apiVersion}/jobs/{job}/log/head

        :param str job: Job ID or UUID. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param int offset: Starting byte from which the file will be read.
        :param int lines: Maximum number of lines to be returned up to a
            maximum of 1000.
        :param str type: Log file to be shown (stdout or stderr).
        """

        return self._get(category='jobs', resource='head', query_id=job, subcategory='log', **options)

    def tail_log(self, job, **options):
        """
        Show the last lines of a log file (up to a limit).
        PATH: /{apiVersion}/jobs/{job}/log/tail

        :param str job: Job ID or UUID. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param int lines: Maximum number of lines to be returned up to a
            maximum of 1000.
        :param str type: Log file to be shown (stdout or stderr).
        """

        return self._get(category='jobs', resource='tail', query_id=job, subcategory='log', **options)




"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Study(_ParentRestClient):
    """
    This class contains methods for the 'Studies' webservices
    PATH: /{apiVersion}/studies
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Study, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def update_acl(self, members, action, data=None, **options):
        """
        Update the set of permissions granted for the member.
        PATH: /{apiVersion}/studies/acl/{members}/update

        :param dict data: JSON containing the parameters to modify ACLs.
            'template' could be either 'admin', 'analyst' or 'view_only'.
            (REQUIRED)
        :param str action: Action to be performed [ADD, SET, REMOVE or RESET].
            Allowed values: ['SET ADD REMOVE RESET'] (REQUIRED)
        :param str members: Comma separated list of user or group ids.
            (REQUIRED)
        """

        options['action'] = action
        return self._post(category='studies', resource='update', subcategory='acl', second_query_id=members, data=data, **options)

    def create(self, data=None, **options):
        """
        Create a new study.
        PATH: /{apiVersion}/studies/create

        :param dict data: study. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str project: Project [organization@]project where project can
            be either the ID or the alias.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._post(category='studies', resource='create', data=data, **options)

    def search(self, project, **options):
        """
        Search studies.
        PATH: /{apiVersion}/studies/search

        :param str project: Project [organization@]project where project can
            be either the ID or the alias. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param str name: Study name.
        :param str id: Study ID.
        :param str alias: Study alias.
        :param str fqn: Study full qualified name.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param str internal_status: Filter by internal status.
        :param str status: Filter by status.
        :param str attributes: Attributes.
        :param str release: Release value.
        """

        options['project'] = project
        return self._get(category='studies', resource='search', **options)

    def acl(self, studies, **options):
        """
        Return the acl of the study. If member is provided, it will only
            return the acl for the member.
        PATH: /{apiVersion}/studies/{studies}/acl

        :param str studies: Comma separated list of Studies
            [[organization@]project:]study where study and project can be
            either the ID or UUID up to a maximum of 100. (REQUIRED)
        :param str member: User or group id.
        :param bool silent: Boolean to retrieve all possible entries that are
            queried for, false to raise an exception whenever one of the
            entries looked for cannot be shown for whichever reason.
        """

        return self._get(category='studies', resource='acl', query_id=studies, **options)

    def info(self, studies, **options):
        """
        Fetch study information.
        PATH: /{apiVersion}/studies/{studies}/info

        :param str studies: Comma separated list of Studies
            [[organization@]project:]study where study and project can be
            either the ID or UUID up to a maximum of 100. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        """

        return self._get(category='studies', resource='info', query_id=studies, **options)

    def search_audit(self, study, **options):
        """
        Search audit collection.
        PATH: /{apiVersion}/studies/{study}/audit/search

        :param str study: Study ID. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param str operation_id: Audit operation UUID.
        :param str user_id: User ID.
        :param str action: Action performed by the user.
        :param str resource: Resource involved. Allowed values: ['AUDIT NOTE
            ORGANIZATION USER PROJECT STUDY FILE SAMPLE JOB INDIVIDUAL COHORT
            DISEASE_PANEL FAMILY CLINICAL_ANALYSIS INTERPRETATION VARIANT
            ALIGNMENT CLINICAL EXPRESSION RGA FUNCTIONAL WORKFLOW RESOURCE']
        :param str resource_id: Resource ID.
        :param str resource_uuid: resource UUID.
        :param str status: Filter by status. Allowed values: ['SUCCESS ERROR']
        :param str date: Date. Format: yyyyMMddHHmmss. Examples: >2018,
            2017-2018, <201805.
        """

        return self._get(category='studies', resource='search', query_id=study, subcategory='audit', **options)

    def groups(self, study, **options):
        """
        Return the groups present in the study. For owners and administrators
            only.
        PATH: /{apiVersion}/studies/{study}/groups

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str id: Group id. If provided, it will only fetch information
            for the provided group.
        :param bool silent: Boolean to retrieve all possible entries that are
            queried for, false to raise an exception whenever one of the
            entries looked for cannot be shown for whichever reason.
        """

        return self._get(category='studies', resource='groups', query_id=study, **options)

    def update_groups(self, study, data=None, **options):
        """
        Add or remove a group.
        PATH: /{apiVersion}/studies/{study}/groups/update

        :param dict data: JSON containing the parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str action: Action to be performed: ADD or REMOVE a group.
            Allowed values: ['ADD REMOVE']
        """

        return self._post(category='studies', resource='update', query_id=study, subcategory='groups', data=data, **options)

    def update_groups_users(self, study, group, data=None, **options):
        """
        Add, set or remove users from an existing group.
        PATH: /{apiVersion}/studies/{study}/groups/{group}/users/update

        :param dict data: JSON containing the parameters. (REQUIRED)
        :param str group: Group name. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str action: Action to be performed: ADD, SET or REMOVE users
            to/from a group. Allowed values: ['ADD SET REMOVE']
        """

        return self._post(category='studies', resource='users/update', query_id=study, subcategory='groups', second_query_id=group, data=data, **options)

    def create_notes(self, study, data=None, **options):
        """
        Create a new note.
        PATH: /{apiVersion}/studies/{study}/notes/create

        :param dict data: JSON containing the Note to be added. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._post(category='studies', resource='create', query_id=study, subcategory='notes', data=data, **options)

    def search_notes(self, study, **options):
        """
        Search for notes of scope STUDY.
        PATH: /{apiVersion}/studies/{study}/notes/search

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param str id: Note unique identifier.
        :param str type: Note type.
        :param str uuid: Unique 32-character identifier assigned automatically
            by OpenCGA.
        :param str user_id: User that wrote that Note.
        :param str tags: Note tags.
        :param str visibility: Visibility of the Note.
        :param str version: Autoincremental version assigned to the registered
            entry. By default, updates does not create new versions. To enable
            versioning, users must set the `incVersion` flag from the /update
            web service when updating the document.
        """

        return self._get(category='studies', resource='search', query_id=study, subcategory='notes', **options)

    def delete_notes(self, study, id, **options):
        """
        Delete note.
        PATH: /{apiVersion}/studies/{study}/notes/{id}/delete

        :param str id: Note unique identifier. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._delete(category='studies', resource='delete', query_id=study, subcategory='notes', second_query_id=id, **options)

    def update_notes(self, study, id, data=None, **options):
        """
        Update a note.
        PATH: /{apiVersion}/studies/{study}/notes/{id}/update

        :param dict data: JSON containing the Note fields to be updated.
            (REQUIRED)
        :param str id: Note unique identifier. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str tags_action: Action to be performed if the array of tags is
            being updated. Allowed values: ['ADD SET REMOVE']
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._post(category='studies', resource='update', query_id=study, subcategory='notes', second_query_id=id, data=data, **options)

    def permission_rules(self, study, entity, **options):
        """
        Fetch permission rules.
        PATH: /{apiVersion}/studies/{study}/permissionRules

        :param str entity: Entity where the permission rules should be applied
            to. Allowed values: ['SAMPLES FILES COHORTS INDIVIDUALS FAMILIES
            JOBS CLINICAL_ANALYSES DISEASE_PANELS'] (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        """

        options['entity'] = entity
        return self._get(category='studies', resource='permissionRules', query_id=study, **options)

    def update_permission_rules(self, study, entity, data=None, **options):
        """
        Add or remove a permission rule.
        PATH: /{apiVersion}/studies/{study}/permissionRules/update

        :param dict data: JSON containing the permission rule to be created or
            removed. (REQUIRED)
        :param str entity: Entity where the permission rules should be applied
            to. Allowed values: ['SAMPLES FILES COHORTS INDIVIDUALS FAMILIES
            JOBS CLINICAL_ANALYSES DISEASE_PANELS'] (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str action: Action to be performed: ADD to add a new permission
            rule; REMOVE to remove all permissions assigned by an existing
            permission rule (even if it overlaps any manual permission); REVERT
            to remove all permissions assigned by an existing permission rule
            (keep manual overlaps); NONE to remove an existing permission rule
            without removing any permissions that could have been assigned
            already by the permission rule. Allowed values: ['ADD REMOVE REVERT
            NONE']
        """

        options['entity'] = entity
        return self._post(category='studies', resource='update', query_id=study, subcategory='permissionRules', data=data, **options)

    def run_templates(self, study, data=None, **options):
        """
        Execute template.
        PATH: /{apiVersion}/studies/{study}/templates/run

        :param dict data: Template loader parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='studies', resource='run', query_id=study, subcategory='templates', data=data, **options)

    def upload_templates(self, study, **options):
        """
        Resource to upload a zipped template.
        PATH: /{apiVersion}/studies/{study}/templates/upload

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param inputstream file: File to upload.
        """

        return self._post(category='studies', resource='upload', query_id=study, subcategory='templates', **options)

    def delete_templates(self, study, template_id, **options):
        """
        Delete template.
        PATH: /{apiVersion}/studies/{study}/templates/{templateId}/delete

        :param str template_id: Template id. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        """

        return self._delete(category='studies', resource='delete', query_id=study, subcategory='templates', second_query_id=template_id, **options)

    def update(self, study, data=None, **options):
        """
        Update some study attributes.
        PATH: /{apiVersion}/studies/{study}/update

        :param dict data: JSON containing the params to be updated. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._post(category='studies', resource='update', query_id=study, data=data, **options)

    def variable_sets(self, study, **options):
        """
        Fetch variableSets from a study.
        PATH: /{apiVersion}/studies/{study}/variableSets

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str id: Id of the variableSet to be retrieved. If no id is
            passed, it will show all the variableSets of the study.
        """

        return self._get(category='studies', resource='variableSets', query_id=study, **options)

    def update_variable_sets(self, study, data=None, **options):
        """
        Add or remove a variableSet.
        PATH: /{apiVersion}/studies/{study}/variableSets/update

        :param dict data: JSON containing the VariableSet to be created or
            removed. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str action: Action to be performed: ADD, REMOVE or FORCE_REMOVE
            a variableSet. Allowed values: ['ADD REMOVE FORCE_REMOVE']
        """

        return self._post(category='studies', resource='update', query_id=study, subcategory='variableSets', data=data, **options)

    def update_variable_sets_variables(self, study, variable_set, data=None, **options):
        """
        Add or remove variables to a VariableSet.
        PATH: /{apiVersion}/studies/{study}/variableSets/{variableSet}/variables/update

        :param dict data: JSON containing the variable to be added or removed.
            For removing, only the variable id will be needed. (REQUIRED)
        :param str variable_set: VariableSet id of the VariableSet to be
            updated. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID. (REQUIRED)
        :param str action: Action to be performed: ADD or REMOVE a variable.
            Allowed values: ['ADD REMOVE']
        """

        return self._post(category='studies', resource='variables/update', query_id=study, subcategory='variableSets', second_query_id=variable_set, data=data, **options)




from pyopencga_enterprise_enterprise.commons import execute
from pyopencga_enterprise_enterprise.rest_response import RestResponse
from pyopencga_enterprise_enterprise.retry import retry


class _ParentRestClient(object):
    """Queries the REST service given the different query params"""

    def __init__(self, configuration, token=None, login_handler=None,
                 auto_refresh=True):
        self.auto_refresh = auto_refresh
        self._cfg = configuration
        self.token = token
        self.refresh_token = None
        self.login_handler = login_handler
        self.on_retry = None

    def _client_login_handler(self):
        if self.login_handler:
            self.token, self.refresh_token = self.login_handler()

    def _refresh_token_client(self):
        if self.login_handler:
            self.token, self.refresh_token = self.login_handler(refresh=True)

    @staticmethod
    def _get_query_id_str(query_ids):
        if query_ids is None:
            return None
        elif isinstance(query_ids, list):
            return ','.join(map(str, query_ids))
        else:
            return str(query_ids)

    def _rest_retry(self, method, category, resource, query_id=None, subcategory=None,
                    second_query_id=None, data=None, dont_retry=None,
                    **options):

        query_ids_str = self._get_query_id_str(query_id)

        def exec_retry():
            return execute(config=self._cfg,
                           sid=self.token,
                           category=category,
                           subcategory=subcategory,
                           method=method,
                           query_id=query_ids_str,
                           second_query_id=second_query_id,
                           resource=resource,
                           data=data,
                           options=options)

        def notify_retry(exc_type, exc_val, exc_tb):
            if self.on_retry is not None:
                self.on_retry(self, exc_type, exc_val, exc_tb, dict(
                    method=method, resource=resource, query_id=query_id,
                    category=category, subcategory=subcategory,
                    second_query_id=second_query_id, data=data,
                    options=options
                ))

        response = retry(
            exec_retry, self._cfg.max_attempts, self._cfg.min_retry_secs,
            self._cfg.max_retry_secs, login_handler=self.login_handler,
            on_retry=notify_retry, dont_retry=dont_retry
        )

        if self.auto_refresh:
            self._refresh_token_client()
        return RestResponse(response)

    def _get(self, category, resource, query_id=None, subcategory=None,
             second_query_id=None, **options):
        """Queries the REST service and returns the result"""
        return self._rest_retry(
            method='get', category=category, resource=resource, query_id=query_id,
            subcategory=subcategory, second_query_id=second_query_id,
            **options
        )

    def _post(self, category, resource, data=None, query_id=None, subcategory=None,
              second_query_id=None, **options):
        """Queries the REST service and returns the result"""
        if data is not None:
            return self._rest_retry(
                method='post', category=category, resource=resource, query_id=query_id,
                subcategory=subcategory, second_query_id=second_query_id,
                data=data, **options
            )
        else:
            return self._rest_retry(
                method='post', category=category, resource=resource, query_id=query_id,
                subcategory=subcategory, second_query_id=second_query_id,
                **options
            )

    def _delete(self, category, resource, query_id=None, subcategory=None,
                second_query_id=None, **options):
        """Queries the REST service and returns the result"""
        return self._rest_retry(
            method='delete', category=category, resource=resource, query_id=query_id,
            subcategory=subcategory, second_query_id=second_query_id,
            **options
        )



import getpass
import time
import sys
import re
if sys.version_info >= (3, 8):
    from importlib.metadata import version
else:
    from importlib_metadata import version

from pyopencga_enterprise_enterprise.opencga_config import ClientConfiguration
from pyopencga_enterprise_enterprise.rest_clients.admin_client import Admin
from pyopencga_enterprise_enterprise.rest_clients.alignment_client import Alignment
from pyopencga_enterprise_enterprise.rest_clients.clinical_analysis_client import ClinicalAnalysis
from pyopencga_enterprise_enterprise.rest_clients.cohort_client import Cohort
from pyopencga_enterprise_enterprise.rest_clients.family_client import Family
from pyopencga_enterprise_enterprise.rest_clients.file_client import File
from pyopencga_enterprise_enterprise.rest_clients.ga4gh_client import GA4GH
from pyopencga_enterprise_enterprise.rest_clients.individual_client import Individual
from pyopencga_enterprise_enterprise.rest_clients.job_client import Job
from pyopencga_enterprise_enterprise.rest_clients.meta_client import Meta
from pyopencga_enterprise_enterprise.rest_clients.disease_panel_client import DiseasePanel
from pyopencga_enterprise_enterprise.rest_clients.project_client import Project
from pyopencga_enterprise_enterprise.rest_clients.sample_client import Sample
from pyopencga_enterprise_enterprise.rest_clients.study_client import Study
from pyopencga_enterprise_enterprise.rest_clients.variant_operation_client import VariantOperation
from pyopencga_enterprise_enterprise.rest_clients.user_client import User
from pyopencga_enterprise_enterprise.rest_clients.variant_client import Variant
from pyopencga_enterprise_enterprise.rest_clients.organization_client import Organization
from pyopencga_enterprise_enterprise.rest_clients.workflow_client import Workflow
from pyopencga_enterprise_enterprise.rest_clients.federation_client import Federation
from pyopencga_enterprise_enterprise.rest_clients.cvdb_client import CVDB


class OpencgaClient(object):
    def __init__(self, configuration, token=None, refresh_token=None, on_retry=None, auto_refresh=True):
        """
        :param on_retry: callback to be called with client retries an operation.
            It must accept parameters: client, exc_type, exc_val, exc_tb, call
        """

        if not isinstance(configuration, ClientConfiguration):
            raise ValueError('Expected ClientConfiguration instance')

        self.configuration = configuration
        self.auto_refresh = auto_refresh
        self.on_retry = on_retry
        self.clients = []
        self._login_handler = None
        self.token = token
        self.refresh_token = refresh_token
        self._create_clients()
        self._check_versions()
        if self.configuration.sso_login:
            self._sso_login()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.logout()

    def _check_versions(self):
        # Getting client and server versions
        client_version = version("pyopencga")
        server_version = self.meta.about().get_result(0)['Version'].split('-')[0]

        ansi_reset = "\033[0m"
        ansi_red = "\033[31m"
        ansi_yellow = "\033[33m"
        if tuple(server_version.split('.')[:2]) < tuple(client_version.split('.')[:2]):
            msg = '[WARNING]: Client version ({}) is higher than server version ({}).' \
                  ' Some client features may not be implemented in the server.\n'.format(client_version, server_version)
            sys.stdout.write('{}{}{}'.format(ansi_red, msg, ansi_reset))
        elif tuple(server_version.split('.')[:2]) > tuple(client_version.split('.')[:2]):
            msg = '[INFO]: Client version ({}) is lower than server version ({}).' \
                   ' Some client features may not work as intended.\n'.format(client_version, server_version)
            sys.stdout.write('{}{}{}'.format(ansi_yellow, msg, ansi_reset))

    def _create_clients(self):
        self.organizations = Organization(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.users = User(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.projects = Project(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.studies = Study(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.files = File(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.jobs = Job(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.samples = Sample(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.individuals = Individual(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.families = Family(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.cohorts = Cohort(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.disease_panels = DiseasePanel(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.alignments = Alignment(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.variants = Variant(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.clinical = ClinicalAnalysis(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.operations = VariantOperation(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.variant_operations = self.operations  # DEPRECATED: use 'self.operations'
        self.meta = Meta(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.ga4gh = GA4GH(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.admin = Admin(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.workflows = Workflow(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.federations = Federation(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)
        self.cvdb = CVDB(self.configuration, self.token, self._login_handler, auto_refresh=self.auto_refresh)

        self.clients = [
            self.organizations, self.users, self.projects, self.studies, self.files, self.jobs,
            self.samples, self.individuals, self.families, self.cohorts,
            self.disease_panels, self.alignments, self.variants, self.clinical, self.operations,
            self.variant_operations, self.meta, self.ga4gh, self.admin, self.workflows, self.federations, self.cvdb
        ]

        for client in self.clients:
            client.on_retry = self.on_retry

    def _make_login_handler(self, user=None, password=None, organization=None):
        """
        Returns a closure that performs the log-in. This will be called on retries
        if the current session ever expires.
        The reason for using a closure and not a normal function is that a normal
        function would require storing the password in a field. It is more secure
        not to do so. This way, the password stored in the closure is inaccessible
        to other code
        """
        def login_handler(refresh=False):
            if refresh:
                data = {'refreshToken': self.refresh_token}
            else:
                data = {'user': user, 'password': password}
            if organization:
                data.update({'organization': organization})
            tokens = User(self.configuration).login(data=data).get_result(0)
            self.token = tokens['token']
            self.refresh_token = tokens['refreshToken']
            return self.token, self.refresh_token
        return login_handler

    def login(self, user=None, password=None, organization=None):
        if user is not None and password is None:
            password = getpass.getpass()
        if not (user and password):
            raise ValueError('User and password required')

        self._login_handler = self._make_login_handler(user=user, password=password, organization=organization)
        self._login_handler()
        for client in self.clients:
            client.token = self.token
            client.refresh_token = self.refresh_token
            client.login_handler = self._login_handler

    def _sso_login(self):
        # Getting token and refresh token from configuration
        self.token = self.configuration.token
        self.refresh_token = self.configuration.token

        self._login_handler = self._make_login_handler()
        for client in self.clients:
            client.token = self.token
            client.refresh_token = self.refresh_token
            client.login_handler = self._login_handler

    def logout(self):
        self.token = None
        self.refresh_token = None
        for client in self.clients:
            client.token = self.token
            client.refresh_token = self.refresh_token

    def wait_for_job(self, response=None, study_id=None, job_id=None, retry_seconds=10):
        if response is not None:
            study_id = response['results'][0]['study']['id']
            job_id = response['results'][0]['id']

        if response is None and (study_id is None or job_id is None):
            raise ValueError('Argument "response" or arguments "study" and "job_id" must be provided')

        if len(job_id.split(',')) > 1:
            raise ValueError('Only one job ID is allowed')

        retry_seconds = retry_seconds if retry_seconds >= 10 else 10
        while True:
            job_info = self.jobs.info(study=study_id, jobs=job_id).get_result(0)
            if job_info['internal']['status']['id'] in ['ERROR', 'ABORTED']:
                raise ValueError('{} ({}): {}'.format(
                    job_info['internal']['status']['id'],
                    job_info['internal']['status']['date'],
                    job_info['internal']['status']['description']
                ))
            elif job_info['internal']['status']['id'] in ['DONE']:
                break
            time.sleep(retry_seconds)

    def _get_help_info(self, client_name=None, parameters=False):
        info = []
        for client in self.clients:
            # Name
            cls_name = type(client).__name__
            client_method = re.sub(r'(?<!^)(?=[A-Z])', '_', cls_name).lower() \
                if cls_name != 'GA4GH' else cls_name.lower()
            client_method = 'get_' + client_method + '_client'

            if client_name is not None and client_name != cls_name:
                continue

            # Description and path
            class_docstring = client.__doc__
            cls_desc = re.findall('(This class contains methods .+)\n', class_docstring)[0]
            cls_path = re.findall('PATH: (.+)\n', class_docstring)[0]

            # Methods
            methods = []
            method_names = [method_name for method_name in dir(client)
                            if callable(getattr(client, method_name))
                            and not method_name.startswith('_') and method_name != 'login_handler']
            for method_name in method_names:
                if client_name is None:
                    continue
                method_docstring = getattr(client, method_name).__doc__
                desc = re.findall('(.+)\n +PATH', method_docstring, re.DOTALL)
                desc = re.sub(' +', ' ', desc[0].replace('\n', ' ').strip())
                path = re.findall('PATH: (.+)\n', method_docstring)[0]

                args = []
                arguments = re.findall(
                    ' +:param (.+)', method_docstring, re.DOTALL
                )
                if arguments and parameters:
                    arguments = arguments[0].replace('\n', ' ').strip()
                    arguments = re.sub(' +', ' ', arguments)
                    arguments = arguments.split(' :param ')
                    for parameter in arguments:
                        param_info = parameter.split(' ', 2)
                        args.append({
                            'name': param_info[1].rstrip(':'),
                            'type': param_info[0],
                            'desc': param_info[2]
                        })
                methods.append({
                    'name': method_name,
                    'desc': desc,
                    'path': path,
                    'params': args
                })

            info.append(
                {'class_name': cls_name, 'client_method': client_method,
                 'desc': cls_desc, 'path': cls_path, 'methods': methods}
            )
        return info

    def help(self, client_name=None, show_parameters=False):
        help_txt = []

        info = self._get_help_info(client_name, show_parameters)
        if client_name is None:
            help_txt += ['Available clients:']
            for client in info:
                txt = '{}- {}: {} ({}). USAGE: opencga_client.{}()'
                help_txt += [txt.format(
                    ' '*4, client['class_name'], client['desc'],
                    client['path'], client['client_method']
                )]
        else:
            for client in info:
                help_txt += ['{}: {} ({}). USAGE: opencga_client.{}()'.format(
                    client['class_name'], client['desc'], client['path'],
                    client['client_method']
                )]
                help_txt += ['{}Available methods:'.format(' '*4)]
                for method in client['methods']:
                    help_txt += ['{}- {}: {} ({})'.format(
                        ' '*8, method['name'], method['desc'], method['path']
                    )]
                    if not show_parameters:
                        continue
                    for param in method['params']:
                        help_txt += ['{}* {} ({}): {}'.format(
                            ' ' * 12, param['name'], param['type'],
                            param['desc']
                        )]
        sys.stdout.write('\n'.join(help_txt) + '\n')

    def get_organization_client(self):
        return self.organizations

    def get_user_client(self):
        return self.users

    def get_project_client(self):
        return self.projects

    def get_study_client(self):
        return self.studies

    def get_file_client(self):
        return self.files

    def get_job_client(self):
        return self.jobs

    def get_sample_client(self):
        return self.samples

    def get_individual_client(self):
        return self.individuals

    def get_family_client(self):
        return self.families

    def get_cohort_client(self):
        return self.cohorts

    def get_disease_panel_client(self):
        return self.disease_panels

    def get_alignment_client(self):
        return self.alignments

    def get_variant_client(self):
        return self.variants

    def get_clinical_client(self):
        return self.clinical

    def get_operations_client(self):
        return self.operations

    def get_variant_operation_client(self):
        return self.variant_operations

    def get_meta_client(self):
        return self.meta

    def get_ga4gh_client(self):
        return self.ga4gh

    def get_admin_client(self):
        return self.admin

    def get_workflows_client(self):
        return self.workflows

    def get_federations_client(self):
        return self.federations

    def get_cvdb_client(self):
        return self.cvdb



"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Admin(_ParentRestClient):
    """
    This class contains methods for the 'Admin' webservices
    PATH: /{apiVersion}/admin
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Admin, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def group_by_audit(self, fields, entity, **options):
        """
        Group by operation.
        PATH: /{apiVersion}/admin/audit/groupBy

        :param str entity: Entity to be grouped by. Allowed values: ['AUDIT
            NOTE ORGANIZATION USER PROJECT STUDY FILE SAMPLE JOB INDIVIDUAL
            COHORT DISEASE_PANEL FAMILY CLINICAL_ANALYSIS INTERPRETATION
            VARIANT ALIGNMENT CLINICAL EXPRESSION RGA FUNCTIONAL WORKFLOW
            RESOURCE'] (REQUIRED)
        :param str fields: Comma separated list of fields by which to group
            by. (REQUIRED)
        :param bool count: Count the number of elements matching the group.
        :param int limit: Maximum number of documents (groups) to be returned.
        :param str action: Action performed.
        :param str before: Object before update.
        :param str after: Object after update.
        :param str date: Date <,<=,>,>=(Format: yyyyMMddHHmmss) and
            yyyyMMddHHmmss-yyyyMMddHHmmss.
        """

        options['fields'] = fields
        options['entity'] = entity
        return self._get(category='admin', resource='groupBy', subcategory='audit', **options)

    def install_catalog(self, data=None, **options):
        """
        Install OpenCGA database.
        PATH: /{apiVersion}/admin/catalog/install

        :param dict data: JSON containing the mandatory parameters. (REQUIRED)
        """

        return self._post(category='admin', resource='install', subcategory='catalog', data=data, **options)

    def jwt_catalog(self, data=None, **options):
        """
        Change JWT secret key.
        PATH: /{apiVersion}/admin/catalog/jwt

        :param dict data: JSON containing the parameters. (REQUIRED)
        :param str organization: Organization id.
        """

        return self._post(category='admin', resource='jwt', subcategory='catalog', data=data, **options)

    def fetch_resource(self, data=None, **options):
        """
        Fetch resources from the public server and save them into the OpenCGA
            local installation.
        PATH: /{apiVersion}/admin/resource/fetch

        :param dict data: Download-resources tool parameters. (REQUIRED)
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='admin', resource='fetch', subcategory='resource', data=data, **options)

    def create_users(self, data=None, **options):
        """
        Create a new user.
        PATH: /{apiVersion}/admin/users/create

        :param dict data: JSON containing the parameters. (REQUIRED)
        """

        return self._post(category='admin', resource='create', subcategory='users', data=data, **options)

    def import_users(self, data=None, **options):
        """
        Import users or a group of users from LDAP or AAD.
        PATH: /{apiVersion}/admin/users/import

        :param dict data: JSON containing the parameters. (REQUIRED)
        :param str organization: Organization id.
        """

        return self._post(category='admin', resource='import', subcategory='users', data=data, **options)

    def permissions_users(self, **options):
        """
        User permissions.
        PATH: /{apiVersion}/admin/users/permissions

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str entry_ids: Comma separated list of entry ids.
        :param str permissions: Comma separated list of permissions to be
            retrieved.
        :param str category: Category corresponding to the id's provided.
        """

        return self._get(category='admin', resource='permissions', subcategory='users', **options)

    def search_users(self, **options):
        """
        User search method.
        PATH: /{apiVersion}/admin/users/search

        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param str organization: Organization id.
        :param str user: User ID.
        :param str authentication_id: Authentication origin ID.
        """

        return self._get(category='admin', resource='search', subcategory='users', **options)

    def sync_users(self, data=None, **options):
        """
        Synchronise a group of users from an authentication origin with a
            group in a study from catalog.
        PATH: /{apiVersion}/admin/users/sync

        :param dict data: JSON containing the parameters. (REQUIRED)
        :param str organization: Organization id.
        """

        return self._post(category='admin', resource='sync', subcategory='users', data=data, **options)

    def users_update_groups(self, user, data=None, **options):
        """
        Add or remove users from existing groups.
        PATH: /{apiVersion}/admin/users/{user}/groups/update

        :param dict data: JSON containing the parameters. (REQUIRED)
        :param str user: User ID. (REQUIRED)
        :param str organization: Organization id.
        :param str action: Action to be performed: ADD or REMOVE user to/from
            groups. Allowed values: ['ADD REMOVE']
        """

        return self._post(category='admin/users', resource='update', query_id=user, subcategory='groups', data=data, **options)




"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Federation(_ParentRestClient):
    """
    This class contains methods for the 'Federations' webservices
    PATH: /{apiVersion}/federations
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Federation, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def connect_client(self, data=None, **options):
        """
        Connect to a shared XetaBase instance.
        PATH: /{apiVersion}/federations/client/connect

        :param dict data: JSON containing the Federation server configuration.
            (REQUIRED)
        """

        return self._post(category='federations', resource='connect', subcategory='client', data=data, **options)

    def client_study_users_list(self, **options):
        """
        Show the list of users with access to the federated study.
        PATH: /{apiVersion}/federations/client/study/users/list

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._get(category='federations', resource='list', subcategory='client/study/users', **options)

    def client_study_users_update(self, data=None, **options):
        """
        Grant/Deny access to federated studies to users.
        PATH: /{apiVersion}/federations/client/study/users/update

        :param dict data: JSON containing the list of users to which this
            action will be applied. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str action: Action to be performed: ADD access or REMOVE
            access. Allowed values: ['ADD', 'REMOVE']
        """

        return self._post(category='federations', resource='update', subcategory='client/study/users', data=data, **options)

    def delete_client(self, id, **options):
        """
        Delete a federation client.
        PATH: /{apiVersion}/federations/client/{id}/delete

        :param str id: Federation client id. (REQUIRED)
        """

        return self._delete(category='federations', resource='delete', subcategory='client', second_query_id=id, **options)

    def synchronize_client(self, id, **options):
        """
        Synchronize data from a known Federation server.
        PATH: /{apiVersion}/federations/client/{id}/synchronize

        :param str id: Federation client id to be synchronized. (REQUIRED)
        """

        return self._post(category='federations', resource='synchronize', subcategory='client', second_query_id=id, **options)

    def update_client(self, id, data=None, **options):
        """
        Update some fields from a Federation client.
        PATH: /{apiVersion}/federations/client/{id}/update

        :param dict data: JSON containing the Federation client parameters to
            be updated. (REQUIRED)
        :param str id: Federation client id. (REQUIRED)
        """

        return self._post(category='federations', resource='update', subcategory='client', second_query_id=id, data=data, **options)

    def create_server(self, data=None, **options):
        """
        Share a resource with another XetaBase instance.
        PATH: /{apiVersion}/federations/server/create

        :param dict data: JSON containing the new Federation object.
            (REQUIRED)
        """

        return self._post(category='federations', resource='create', subcategory='server', data=data, **options)

    def delete_server(self, id, **options):
        """
        Delete a federation server.
        PATH: /{apiVersion}/federations/server/{id}/delete

        :param str id: Federation server id. (REQUIRED)
        """

        return self._delete(category='federations', resource='delete', subcategory='server', second_query_id=id, **options)

    def reset_server(self, id, **options):
        """
        Reset the credentials of a federation server.
        PATH: /{apiVersion}/federations/server/{id}/reset

        :param str id: Federation server id to reset. (REQUIRED)
        """

        return self._post(category='federations', resource='reset', subcategory='server', second_query_id=id, **options)

    def update_server(self, id, data=None, **options):
        """
        Update some fields from a Federation server.
        PATH: /{apiVersion}/federations/server/{id}/update

        :param dict data: JSON containing the Federation server parameters to
            be updated. (REQUIRED)
        :param str id: Federation server id. (REQUIRED)
        """

        return self._post(category='federations', resource='update', subcategory='server', second_query_id=id, data=data, **options)

